;(function($, window, document, undefined) {
	$.fn.equalizeHeight = function(settings) {
		'use strict';

		var $doc = $(document);
		var $win = $(window);
		var EQ = this;
		var offsets = [];
		var elementsByOffset = {};
		var maxHeight = 0;

		function addOffset(offset) {
			if ( offsets.indexOf(offset) < 0 ) {
				offsets.push(offset);
			};
		};

		function setHeight(collection) {
			var maximumHeight = 0;
			
			$.each(collection, function(i, element) {
				$(element).height('auto');

				if ( $(element).height() > maximumHeight ) {
					maximumHeight = $(element).height();
				}
			});

			if ( $win.width() > EQ.options.breakpoint ) {
				for (var i = collection.length - 1; i >= 0; i--) {
					$(collection[i]).height(maximumHeight)
				};
			};
		};

		EQ.options = $.extend({
			selector: '.equalize',
			byRow: true,
			breakpoint: 767
		}, settings);

		EQ.selector = EQ.options.selector;
		EQ.byRow = EQ.options.byRow;
		EQ.breakpoint = EQ.options.breakpoint;

		$win.on('load.eq resize.eq', function() {
			EQ.init();
		});

		EQ.init = function() {
			EQ.bind();
		};
		
		EQ.bind = function() {
			EQ.updateOffsets();
			EQ.pushElements();
			EQ.equalize();
		};

		EQ.updateOffsets = function() {
			var baseOffset = $(EQ.selector).eq(0).offset().top;

			offsets = [];
			elementsByOffset = {};

			addOffset(baseOffset);
			elementsByOffset[baseOffset] = [];

			$(EQ.selector).each(function() {
				var $element = $(this);

				if ( $element.offset().top !== baseOffset ) {
					baseOffset = $element.offset().top;
					addOffset(baseOffset);
					elementsByOffset[baseOffset] = [];
				};
			});
		};

		EQ.pushElements = function() {
			var offsetsLen = offsets.length;
			var i = 0;

			for ( ; i < offsetsLen; i++ ) {
				$(EQ.selector).each(function() {
					if ( $(this).offset().top === offsets[i] ) {
						elementsByOffset[offsets[i]].push(this);
					};
				});
			};
		};

		EQ.equalize = function() {
			if ( !EQ.options.byRow ) {
				if ( $win.width() > EQ.options.breakpoint ) {
					$(EQ.selector)
						.height('auto')
						.each(function() {
							var $element = $(this);

							if ( $element.height() > maxHeight ) {
								maxHeight = $element.height();
							};
						})
						.height(maxHeight);
				} else {
					$(EQ.selector)
						.height('auto');
				};
			} else {
				for ( var offset in elementsByOffset ) {
					if ( elementsByOffset.hasOwnProperty(offset) ) {
						setHeight(elementsByOffset[offset]);
					};
				};
			};
		};
	};

	$(document).ready(function() {
		$('.equalize').equalizeHeight({
			selector: '.equalcol',
			byRow: true,
			breakpoint: 767
		});
	});
})(jQuery, window, document);
